package com.lasalle.mqttmosquitto;

import android.content.Context;
import android.util.Log;

import org.eclipse.paho.android.service.MqttAndroidClient;
import org.eclipse.paho.client.mqttv3.DisconnectedBufferOptions;
import org.eclipse.paho.client.mqttv3.IMqttActionListener;
import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.IMqttToken;
import org.eclipse.paho.client.mqttv3.MqttCallbackExtended;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;


public class ClientMQTT
{
    private static final String TAG = "ClientMQTT";
    Context context;
    public MqttAndroidClient mqttAndroidClient = null;
    String serverUri = "";
    int portMosquitto = 1883;
    String clientId = "";
    String subscriptionTopicUp = "";
    String publishTopicDown = "";
    // Pour les tests :
    String brokerMosquitto = "192.168.52.36";  // broker mosquitto
    String applicationId = "eco";
    String deviceIdUp = "temperature";
    String deviceIdDown = "batterie";

    public ClientMQTT(Context context)
    {
        this.context = context;

        this.serverUri = "tcp://" + brokerMosquitto + ":" + portMosquitto;
        this.clientId = applicationId;
        this.subscriptionTopicUp = applicationId + "/devices/" + deviceIdUp;
        this.publishTopicDown = applicationId + "/devices/" + deviceIdDown;

        Log.w(TAG, "MqttAndroidClient : serverUri -> " + serverUri);
        Log.w(TAG, "MqttAndroidClient : applicationId -> " + applicationId);
        Log.w(TAG, "MqttAndroidClient : deviceId -> " + deviceIdUp);
        Log.w(TAG, "MqttAndroidClient : subscriptionTopic -> " + subscriptionTopicUp);
        Log.w(TAG, "MqttAndroidClient : deviceId -> " + deviceIdDown);
        Log.w(TAG, "MqttAndroidClient : publishTopic -> " + publishTopicDown);

        creer();
    }

    public ClientMQTT(Context context, String brokerMosquitto, int portMosquitto, String applicationId)
    {
        this.context = context;
        this.serverUri = "tcp://" + brokerMosquitto + ":" + portMosquitto;
        this.portMosquitto = portMosquitto;
        this.clientId = applicationId;
        this.subscriptionTopicUp = applicationId + "/devices/" + deviceIdUp;
        this.publishTopicDown = applicationId + "/devices/" + deviceIdDown;

        Log.w(TAG, "MqttAndroidClient : serverUri -> " + serverUri);
        Log.w(TAG, "MqttAndroidClient : applicationId -> " + applicationId);
        Log.w(TAG, "MqttAndroidClient : deviceId -> " + deviceIdUp);
        Log.w(TAG, "MqttAndroidClient : subscriptionTopic -> " + subscriptionTopicUp);
        Log.w(TAG, "MqttAndroidClient : deviceId -> " + deviceIdDown);
        Log.w(TAG, "MqttAndroidClient : publishTopic -> " + publishTopicDown);

        creer();
    }

    public String getServerUri()
    {
        return serverUri;
    }

    public void creer()
    {
        Log.w(TAG, "MqttAndroidClient.creer()");
        Log.w(TAG, "MqttAndroidClient : serverUri -> " + serverUri);
        Log.w(TAG, "MqttAndroidClient : clientId -> " + clientId);
        mqttAndroidClient = new MqttAndroidClient(context, serverUri, clientId);
        mqttAndroidClient.setCallback(new MqttCallbackExtended()
        {
            @Override
            public void connectComplete(boolean b, String s)
            {
                Log.w(TAG, "MqttAndroidClient : connectComplete -> " + s + " (" + b + ")");
            }

            @Override
            public void connectionLost(Throwable throwable)
            {
                Log.w(TAG, "MqttAndroidClient : connectionLost");
            }

            @Override
            public void messageArrived(String topic, MqttMessage mqttMessage) throws Exception
            {
                Log.w(TAG, "MqttAndroidClient : messageArrived -> " + mqttMessage.toString());
            }

            @Override
            public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken)
            {
                Log.w(TAG, "MqttAndroidClient : deliveryComplete");
            }
        });
    }

    public void setCallback(MqttCallbackExtended callback)
    {
        mqttAndroidClient.setCallback(callback);
    }

    public void connecter()
    {
        Log.w(TAG, "MqttAndroidClient.connecter()");
        MqttConnectOptions mqttConnectOptions = new MqttConnectOptions();
        mqttConnectOptions.setAutomaticReconnect(true);
        mqttConnectOptions.setCleanSession(false);

        try {
            mqttAndroidClient.connect(mqttConnectOptions, null, new IMqttActionListener()
            {
                @Override
                public void onSuccess(IMqttToken asyncActionToken)
                {
                    Log.w(TAG, "connect() : onSuccess");
                    DisconnectedBufferOptions disconnectedBufferOptions = new DisconnectedBufferOptions();
                    disconnectedBufferOptions.setBufferEnabled(true);
                    disconnectedBufferOptions.setBufferSize(100);
                    disconnectedBufferOptions.setPersistBuffer(false);
                    disconnectedBufferOptions.setDeleteOldestMessages(false);
                    mqttAndroidClient.setBufferOpts(disconnectedBufferOptions);
                    souscrire();
                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception)
                {
                    Log.w(TAG, "connect() : onFailure -> " + serverUri + exception.toString());
                }
            });
        }
        catch (MqttException ex)
        {
            ex.printStackTrace();
            Log.e(TAG, "connect() : exception");
        }
    }

    public void reconnecter()
    {
        Log.w(TAG, "MqttAndroidClient.reconnecter()");
        if(estConnecte())
            deconnecter();
        connecter();
    }

    public void deconnecter()
    {
        Log.w(TAG, "MqttAndroidClient.deconnecter()");
        Thread deconnexion = new Thread(new Runnable()
        {
            public void run()
            {
                try
                {
                    mqttAndroidClient.disconnect();
                }
                catch (MqttException e)
                {
                    e.printStackTrace();
                    Log.e(TAG, "disconnect() -> exception");
                }
            }
        });
        // Démarrage
        deconnexion.start();
    }

    public boolean estConnecte()
    {
        Log.w(TAG, "MqttAndroidClient.estConnecte() -> " + mqttAndroidClient.isConnected());
        return mqttAndroidClient.isConnected();
    }

    public boolean publier(String donnee)
    {
        if(estConnecte())
        {
            //MqttMessage mqttMessage = new MqttMessage(intToByteArray(niveau));
            MqttMessage mqttMessage = new MqttMessage(donnee.getBytes());
            try
            {
                Log.w(TAG, "MqttAndroidClient.publier() -> " + publishTopicDown);
                Log.w(TAG, "MqttAndroidClient.publier() -> " + donnee);
                mqttAndroidClient.publish(publishTopicDown, mqttMessage);
                return true;
            }
            catch(MqttException e)
            {
                e.printStackTrace();
            }
        }
        return false;
    }

    public boolean souscrire(String topic)
    {
        Log.w(TAG, "MqttAndroidClient.souscrire() -> " + topic);
        try
        {
            final boolean[] retour = {false};
            mqttAndroidClient.subscribe(topic, 0, null, new IMqttActionListener()
            {
                @Override
                public void onSuccess(IMqttToken asyncActionToken)
                {
                    Log.w(TAG,"subscribe() : onSuccess");
                    retour[0] = true;
                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception)
                {
                    Log.w(TAG, "subscribe() : onFailure");
                    retour[0] = false;
                }
            });
            return retour[0];
        }
        catch (MqttException ex)
        {
            Log.e(TAG, "subscribe() : exception");
            ex.printStackTrace();
            return false;
        }
    }

    private void souscrire()
    {
        try
        {
            mqttAndroidClient.subscribe(subscriptionTopicUp, 0, null, new IMqttActionListener()
            {
                @Override
                public void onSuccess(IMqttToken asyncActionToken)
                {
                    Log.w(TAG,"subscribe() : onSuccess");
                }

                @Override
                public void onFailure(IMqttToken asyncActionToken, Throwable exception)
                {
                    Log.w(TAG, "subscribe() : onFailure");
                }
            });

        }
        catch (MqttException ex)
        {
            Log.e(TAG, "subscribe() : exception");
            ex.printStackTrace();
        }
    }

    public void setParametres(String brokerMosquitto, int portMosquitto, String applicationId)
    {
        this.serverUri = "tcp://" + brokerMosquitto + ":" + portMosquitto;
        this.portMosquitto = portMosquitto;
        this.clientId = applicationId;

        Log.w(TAG, "MqttAndroidClient : serverUri -> " + serverUri);
        Log.w(TAG, "MqttAndroidClient : applicationId -> " + applicationId);
    }

    public static byte[] intToByteArray(int a)
    {
        byte[] ret = new byte[4];
        ret[3] = (byte) (a & 0xFF);
        ret[2] = (byte) ((a >> 8) & 0xFF);
        ret[1] = (byte) ((a >> 16) & 0xFF);
        ret[0] = (byte) ((a >> 24) & 0xFF);
        return ret;
    }
}
