package com.lasalle.mqttmosquitto;

import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.BatteryManager;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.TextView;

import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallbackExtended;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;

public class MainActivity extends AppCompatActivity
{
    private static final String TAG = "MainActivity"; //!< le TAG de la classe pour les logs
    final int ID_Intent_ParametresConnexion = 1; //!< l'ID de l'Intent ParametresConnexion
    ClientMQTT clientMQTT = null;
    // GUI
    MenuItem menuConnexion = null;
    TextView txtNiveauBatterie;
    TextView txtEtatConnexion;
    TextView txtTopic;
    TextView txtResultat;
    // Pour les tests :
    String brokerMosquitto = "192.168.52.36";  // broker mosquitto
    int portMosquitto = 1883;
    String applicationId = "eco";
    String deviceId = "temperature";

    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        // accès aux éléments de l'IHM
        txtNiveauBatterie = (TextView) this.findViewById(R.id.txtNiveauBatterie);
        txtEtatConnexion = (TextView) this.findViewById(R.id.txtEtatConnexion);
        txtTopic = (TextView) this.findViewById(R.id.txtTopic);
        txtResultat = (TextView) this.findViewById(R.id.txtResultat);

        clientMQTT = new ClientMQTT(getApplicationContext());

        // Enregistre le listener sur l'évènement Changement d'état de la batterie
        this.registerReceiver(this.batterie, new IntentFilter(Intent.ACTION_BATTERY_CHANGED));
    }

    @Override
    public void onDestroy()
    {
        super.onDestroy();
        this.unregisterReceiver(this.batterie);
    }

    private void demarrerMQTT()
    {
        clientMQTT.reconnecter();

        clientMQTT.mqttAndroidClient.setCallback(new MqttCallbackExtended()
        {
            @Override
            public void connectComplete(boolean b, String s)
            {
                Log.w(TAG,"connectComplete");
                afficherEtatConnexion("Mosquitto connecté : " + clientMQTT.getServerUri());
                if(menuConnexion != null)
                    menuConnexion.setTitle("Déconnecter");
            }

            @Override
            public void connectionLost(Throwable throwable)
            {
                Log.w(TAG,"connectionLost");
                afficherEtatConnexion("Mosquitto déconnecté : " + clientMQTT.getServerUri());
                if(menuConnexion != null)
                    menuConnexion.setTitle("Connecter");
            }

            @Override
            public void messageArrived(String topic, MqttMessage mqttMessage) throws Exception
            {
                Log.w(TAG, "messageArrived : " + topic);
                Log.w(TAG, "messageArrived : " + mqttMessage.toString());
                afficherResultat(topic, mqttMessage.toString());
            }

            @Override
            public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken)
            {
                Log.w(TAG, "deliveryComplete");
            }
        });
    }

    private void arreterMQTT()
    {
        if(clientMQTT.estConnecte())
            clientMQTT.deconnecter();
    }

    /**
     * @fn afficherEtatConnexion
     * @brief Affichage l'état de la connexion MySQL
     */
    public void afficherEtatConnexion(String message)
    {
        txtEtatConnexion.setText(message);
        txtTopic.setText("");
        txtResultat.setText("");
    }

    /**
     * @fn afficherResultat
     * @brief Affichage du résultat
     */
    public void afficherResultat(String topic, String resultat)
    {
        txtTopic.setText(topic);
        txtResultat.setText(resultat);
    }

    /**
     * @fn parametrer
     * @brief Paramétrage de la connexion
     */
    private void parametrer()
    {
        // Crée et démarre une activité
        Intent intent = new Intent(MainActivity.this, ParametresConnexionActivity.class);
        // Passage de données
        intent.putExtra("brokerMosquitto", brokerMosquitto);
        intent.putExtra("portMosquitto", portMosquitto);
        intent.putExtra("applicationId", applicationId);
        //startActivity(intent);
        startActivityForResult(intent, ID_Intent_ParametresConnexion);
    }

    /**
     * @fn onActivityResult
     * @brief
     */
    protected void onActivityResult(int requestCode, int resultCode, Intent intent)
    {
        Log.d(TAG, "requestCode=" + requestCode);
        Log.d(TAG, "resultCode=" + resultCode);
        if (requestCode == ID_Intent_ParametresConnexion)
        {
            //if(resultCode == RESULT_CANCELED)
            //...
            if (resultCode == RESULT_OK)
            {
                brokerMosquitto = intent.getStringExtra("brokerMosquitto");
                portMosquitto = intent.getIntExtra("portMosquitto", portMosquitto);
                applicationId = intent.getStringExtra("applicationId");
                deviceId = intent.getStringExtra("deviceId");

                Log.d(TAG,"brokerMosquitto : " + brokerMosquitto);
                Log.d(TAG,"portTTNportMosquitto : " + portMosquitto);
                Log.d(TAG,"applicationId : " + applicationId);
                Log.d(TAG,"deviceId : " + deviceId);

                clientMQTT.setParametres(brokerMosquitto, portMosquitto, applicationId);
            }
        }
    }

    /**
     * @fn onCreateOptionsMenu
     * @brief Création du menu Options (en haut à droite)
     */
    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    /**
     * @fn onOptionsItemSelected
     * @brief Sélection d'une entrée du du menu Options (en haut à droite)
     */
    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        Log.d(TAG, "onOptionsItemSelected -> titre : " + item.getTitle());

        if(menuConnexion == null)
            menuConnexion = item;
        int id = item.getItemId();

        switch(id)
        {
            case R.id.action_parametres:
                parametrer();
                break;
            case R.id.action_connexion:
                if(item.getTitle().toString().equals("Connecter"))
                {
                    demarrerMQTT();
                    if(clientMQTT.estConnecte())
                        item.setTitle("Déconnecter");
                }
                else if(item.getTitle().toString().equals("Déconnecter"))
                {
                    arreterMQTT();
                    item.setTitle("Connecter");
                    afficherEtatConnexion("Déconnecté");
                }
                break;
        }

        return super.onOptionsItemSelected(item);
    }

    private BroadcastReceiver batterie = new BroadcastReceiver()
    {
        @Override
        public void onReceive(Context context, Intent intent)
        {
            // les propriétés de la batterie
            int sante = intent.getIntExtra(BatteryManager.EXTRA_HEALTH, 0);
            int niveau = intent.getIntExtra(BatteryManager.EXTRA_LEVEL, 0);
            int echelle = intent.getIntExtra(BatteryManager.EXTRA_SCALE, 0);
            int typeAlimentation = intent.getIntExtra(BatteryManager.EXTRA_PLUGGED, -1);
            String technologie = intent.getExtras().getString(BatteryManager.EXTRA_TECHNOLOGY);
            int temperature = intent.getIntExtra(BatteryManager.EXTRA_TEMPERATURE, 0);
            int tension = intent.getIntExtra(BatteryManager.EXTRA_VOLTAGE, 0);
            // voir aussi
            /*
            int icone = intent.getIntExtra(BatteryManager.EXTRA_ICON_SMALL, 0);
            int etat = intent.getIntExtra(BatteryManager.EXTRA_STATUS, 0);
            boolean estPresente = intent.getExtras().getBoolean(BatteryManager.EXTRA_PRESENT);
            */

            // type d'alimentation
            //Log.d(TAG,"Batterie : EXTRA_PLUGGED -> " + typeAlimentation);
            switch(typeAlimentation)
            {
                case 0:
                    Log.v(TAG,"Batterie : pas en charge");
                    break;
                case BatteryManager.BATTERY_PLUGGED_AC:
                    Log.v(TAG,"Batterie : en charge secteur");
                    break;
                case BatteryManager.BATTERY_PLUGGED_USB:
                    Log.v(TAG,"Batterie : en charge usb");
                    break;
            }

            Log.v(TAG,"Batterie : " + String.valueOf((float) tension / 1000) + " V");
            Log.v(TAG,"Batterie : " + String.valueOf((float) temperature / 10) + " °C");
            Log.v(TAG,"Batterie : " + String.valueOf(niveau) + " %");

            switch(sante)
            {
                case BatteryManager.BATTERY_HEALTH_DEAD:
                    Log.v(TAG,"Batterie : morte");
                    break;
                case BatteryManager.BATTERY_HEALTH_GOOD:
                    Log.v(TAG,"Batterie : bonne");
                    break;
                case BatteryManager.BATTERY_HEALTH_OVERHEAT:
                    Log.v(TAG,"Batterie : en surchauffe");
                    break;
                case BatteryManager.BATTERY_HEALTH_OVER_VOLTAGE:
                    Log.v(TAG,"Batterie : surtension");
                    break;
            }

            txtNiveauBatterie.setText("Batterie : " + String.valueOf(niveau) + " %");
            clientMQTT.publier(String.valueOf(niveau));
        }
    };
}